@@ -72,6 +72,7 @@
 #include "feature/stats/predict_ports.h"
 #include "lib/crypt_ops/crypto_rand.h"
 #include "lib/trace/events.h"
+#include "core/or/congestion_control_common.h"
 
 #include "core/or/cell_st.h"
 #include "core/or/cpath_build_state_st.h"
@@ -81,6 +82,9 @@
 #include "core/or/or_circuit_st.h"
 #include "core/or/origin_circuit_st.h"
 
+#include "trunnel/extension.h"
+#include "trunnel/congestion_control.h"
+
 static int circuit_send_first_onion_skin(origin_circuit_t *circ);
 static int circuit_build_no_more_hops(origin_circuit_t *circ);
 static int circuit_send_intermediate_onion_skin(origin_circuit_t *circ,
@@ -841,7 +845,13 @@ circuit_pick_create_handshake(uint8_t *cell_type_out,
    * using the TAP handshake, and CREATE2 otherwise. */
   if (extend_info_supports_ntor(ei)) {
     *cell_type_out = CELL_CREATE2;
-    *handshake_type_out = ONION_HANDSHAKE_TYPE_NTOR;
+    /* Only use ntor v3 with exits that support congestion control,
+     * and only when it is enabled. */
+    if (ei->exit_supports_congestion_control &&
+        congestion_control_enabled())
+      *handshake_type_out = ONION_HANDSHAKE_TYPE_NTOR_V3;
+    else
+      *handshake_type_out = ONION_HANDSHAKE_TYPE_NTOR;
   } else {
     /* XXXX030 Remove support for deciding to use TAP and EXTEND. */
     *cell_type_out = CELL_CREATE;
@@ -995,7 +1005,8 @@ circuit_send_first_onion_skin(origin_circuit_t *circ)
   len = onion_skin_create(cc.handshake_type,
                           circ->cpath->extend_info,
                           &circ->cpath->handshake_state,
-                          cc.onionskin);
+                          cc.onionskin,
+                          sizeof(cc.onionskin));
   if (len < 0) {
     log_warn(LD_CIRC,"onion_skin_create (first hop) failed.");
     return - END_CIRC_REASON_INTERNAL;
@@ -1142,7 +1153,8 @@ circuit_send_intermediate_onion_skin(origin_circuit_t *circ,
   len = onion_skin_create(ec.create_cell.handshake_type,
                           hop->extend_info,
                           &hop->handshake_state,
-                          ec.create_cell.onionskin);
+                          ec.create_cell.onionskin,
+                          sizeof(ec.create_cell.onionskin));
   if (len < 0) {
     log_warn(LD_CIRC,"onion_skin_create failed.");
     return - END_CIRC_REASON_INTERNAL;
@@ -1240,6 +1252,7 @@ circuit_finish_handshake(origin_circuit_t *circ,
   }
   tor_assert(hop->state == CPATH_STATE_AWAITING_KEYS);
 
+  circuit_params_t params;
   {
     const char *msg = NULL;
     if (onion_skin_client_handshake(hop->handshake_state.tag,
@@ -1247,6 +1260,7 @@ circuit_finish_handshake(origin_circuit_t *circ,
                                     reply->reply, reply->handshake_len,
                                     (uint8_t*)keys, sizeof(keys),
                                     (uint8_t*)hop->rend_circ_nonce,
+                                    &params,
                                     &msg) < 0) {
       if (msg)
         log_warn(LD_CIRC,"onion_skin_client_handshake failed: %s", msg);
@@ -1260,6 +1274,24 @@ circuit_finish_handshake(origin_circuit_t *circ,
     return -END_CIRC_REASON_TORPROTOCOL;
   }
 
+  if (params.cc_enabled) {
+    int circ_len = circuit_get_cpath_len(circ);
+
+    if (circ_len == DEFAULT_ROUTE_LEN &&
+        circuit_get_cpath_hop(circ, DEFAULT_ROUTE_LEN) == hop) {
+      hop->ccontrol = congestion_control_new(&params, CC_PATH_EXIT);
+    } else if (circ_len == SBWS_ROUTE_LEN &&
+               circuit_get_cpath_hop(circ, SBWS_ROUTE_LEN) == hop) {
+      hop->ccontrol = congestion_control_new(&params, CC_PATH_SBWS);
+    } else {
+      static ratelim_t cc_path_limit = RATELIM_INIT(600);
+      log_fn_ratelim(&cc_path_limit, LOG_WARN, LD_CIRC,
+                     "Unexpected path length %d for circuit",
+                     circ_len);
+      hop->ccontrol = congestion_control_new(&params, CC_PATH_EXIT);
+    }
+  }
+
   hop->state = CPATH_STATE_OPEN;
   log_info(LD_CIRC,"Finished building circuit hop:");
   circuit_log_path(LOG_INFO,LD_CIRC,circ);
@@ -2059,7 +2091,10 @@ onion_pick_cpath_exit(origin_circuit_t *circ, extend_info_t *exit_ei,
       log_warn(LD_CIRC,"Failed to choose an exit server");
       return -1;
     }
-    exit_ei = extend_info_from_node(node, state->onehop_tunnel);
+    exit_ei = extend_info_from_node(node, state->onehop_tunnel,
+                /* for_exit_use */
+                !state->is_internal && TO_CIRCUIT(circ)->purpose ==
+                  CIRCUIT_PURPOSE_C_GENERAL);
     if (BUG(exit_ei == NULL))
       return -1;
   }
@@ -2455,7 +2490,7 @@ onion_extend_cpath(origin_circuit_t *circ)
          primary address, for potentially connecting to an IPv6 OR
          port. Servers always want the primary (IPv4) address. */
       int client = (server_mode(get_options()) == 0);
-      info = extend_info_from_node(r, client);
+      info = extend_info_from_node(r, client, false);
       /* Clients can fail to find an allowed address */
       tor_assert_nonfatal(info || client);
     }
@@ -2463,7 +2498,7 @@ onion_extend_cpath(origin_circuit_t *circ)
     const node_t *r =
       choose_good_middle_server(purpose, state, circ->cpath, cur_len);
     if (r) {
-      info = extend_info_from_node(r, 0);
+      info = extend_info_from_node(r, 0, false);
     }
   }
 
@@ -2573,3 +2608,25 @@ circuit_upgrade_circuits_from_guard_wait(void)
 
   smartlist_free(to_upgrade);
 }
+
+/**
+ * Try to generate a circuit-negotiation message for communication with a
+ * given relay.  Assumes we are using ntor v3, or some later version that
+ * supports parameter negotiatoin.
+ *
+ * On success, return 0 and pass back a message in the `out` parameters.
+ * Otherwise, return -1.
+ **/
+int
+client_circ_negotiation_message(const extend_info_t *ei,
+                                uint8_t **msg_out,
+                                size_t *msg_len_out)
+{
+  tor_assert(ei && msg_out && msg_len_out);
+
+  if (!ei->exit_supports_congestion_control) {
+    return -1;
+  }
+
+  return congestion_control_build_ext_request(msg_out, msg_len_out);
+}
