@@ -10,6 +10,7 @@
 
 #include "core/or/or.h"
 
+#include "core/crypto/onion_crypto.h"
 #include "core/or/circuitlist.h"
 #include "core/or/crypt_path.h"
 #include "core/or/or_circuit_st.h"
@@ -25,22 +26,32 @@
 #include "core/or/trace_probes_cc.h"
 #include "lib/time/compat_time.h"
 #include "feature/nodelist/networkstatus.h"
+#include "app/config/config.h"
+
+#include "trunnel/congestion_control.h"
+#include "trunnel/extension.h"
 
 /* Consensus parameter defaults.
  *
  * More details for each of the parameters can be found in proposal 324,
  * section 6.5 including tuning notes. */
-#define CIRCWINDOW_INIT (500)
-#define SENDME_INC_DFLT (50)
+#define SENDME_INC_DFLT (TLS_RECORD_MAX_CELLS)
+#define CIRCWINDOW_INIT (4*SENDME_INC_DFLT)
 
-#define CWND_INC_DFLT (50)
-#define CWND_INC_PCT_SS_DFLT (100)
+#define CC_ALG_DFLT (CC_ALG_SENDME)
+#define CC_ALG_DFLT_ALWAYS (CC_ALG_VEGAS)
+
+#define CWND_INC_DFLT (TLS_RECORD_MAX_CELLS)
+#define CWND_INC_PCT_SS_DFLT (50)
 #define CWND_INC_RATE_DFLT (1)
+
+#define CWND_MIN_DFLT (SENDME_INC_DFLT)
 #define CWND_MAX_DFLT (INT32_MAX)
-#define CWND_MIN_DFLT (MAX(100, SENDME_INC_DFLT))
 
 #define BWE_SENDME_MIN_DFLT (5)
-#define EWMA_CWND_COUNT_DFLT (2)
+
+#define N_EWMA_CWND_PCT_DFLT (50)
+#define N_EWMA_MAX_DFLT (10)
 
 /* BDP algorithms for each congestion control algorithms use the piecewise
  * estimattor. See section 3.1.4 of proposal 324. */
@@ -74,6 +85,8 @@ static bool congestion_control_update_circuit_bdp(congestion_control_t *,
                                                   const circuit_t *,
                                                   const crypt_path_t *,
                                                   uint64_t, uint64_t);
+/* For unit tests */
+void congestion_control_set_cc_enabled(void);
 
 /* Consensus parameters cached. The non static ones are extern. */
 static uint32_t cwnd_max = CWND_MAX_DFLT;
@@ -81,6 +94,23 @@ int32_t cell_queue_high = CELL_QUEUE_HIGH_DFLT;
 int32_t cell_queue_low = CELL_QUEUE_LOW_DFLT;
 uint32_t or_conn_highwater = OR_CONN_HIGHWATER_DFLT;
 uint32_t or_conn_lowwater = OR_CONN_LOWWATER_DFLT;
+uint8_t cc_sendme_inc = SENDME_INC_DFLT;
+static cc_alg_t cc_alg = CC_ALG_DFLT;
+
+/**
+ * Number of cwnd worth of sendme acks to smooth RTT and BDP with,
+ * using N_EWMA */
+static uint8_t n_ewma_cwnd_pct;
+
+/**
+ * Maximum number N for the N-count EWMA averaging of RTT and BDP.
+ */
+static uint8_t n_ewma_max;
+
+/**
+ * Minimum number of sendmes before we begin BDP estimates
+ */
+static uint8_t bwe_sendme_min;
 
 /**
  * Update global congestion control related consensus parameter values,
@@ -126,6 +156,46 @@ congestion_control_new_consensus_params(const networkstatus_t *ns)
         CWND_MAX_DFLT,
         CWND_MAX_MIN,
         CWND_MAX_MAX);
+
+#define SENDME_INC_MIN 1
+#define SENDME_INC_MAX (255)
+  cc_sendme_inc =
+    networkstatus_get_param(NULL, "cc_sendme_inc",
+        SENDME_INC_DFLT,
+        SENDME_INC_MIN,
+        SENDME_INC_MAX);
+
+#define CC_ALG_MIN 0
+#define CC_ALG_MAX (NUM_CC_ALGS-1)
+  cc_alg =
+    networkstatus_get_param(NULL, "cc_alg",
+        CC_ALG_DFLT,
+        CC_ALG_MIN,
+        CC_ALG_MAX);
+
+#define BWE_SENDME_MIN_MIN 2
+#define BWE_SENDME_MIN_MAX (20)
+  bwe_sendme_min =
+    networkstatus_get_param(NULL, "cc_bwe_min",
+        BWE_SENDME_MIN_DFLT,
+        BWE_SENDME_MIN_MIN,
+        BWE_SENDME_MIN_MAX);
+
+#define N_EWMA_CWND_PCT_MIN 1
+#define N_EWMA_CWND_PCT_MAX (255)
+  n_ewma_cwnd_pct =
+    networkstatus_get_param(NULL, "cc_ewma_cwnd_pct",
+        N_EWMA_CWND_PCT_DFLT,
+        N_EWMA_CWND_PCT_MIN,
+        N_EWMA_CWND_PCT_MAX);
+
+#define N_EWMA_MAX_MIN 2
+#define N_EWMA_MAX_MAX (INT32_MAX)
+  n_ewma_max =
+    networkstatus_get_param(NULL, "cc_ewma_max",
+        N_EWMA_MAX_DFLT,
+        N_EWMA_MAX_MIN,
+        N_EWMA_MAX_MAX);
 }
 
 /**
@@ -139,10 +209,13 @@ congestion_control_new_consensus_params(const networkstatus_t *ns)
  */
 static void
 congestion_control_init_params(congestion_control_t *cc,
-                               cc_alg_t cc_alg,
-                               int sendme_inc)
+                               const circuit_params_t *params,
+                               cc_path_t path)
 {
-#define CWND_INIT_MIN 100
+  const or_options_t *opts = get_options();
+  cc->sendme_inc = params->sendme_inc_cells;
+
+#define CWND_INIT_MIN SENDME_INC_DFLT
 #define CWND_INIT_MAX (10000)
   cc->cwnd =
     networkstatus_get_param(NULL, "cc_cwnd_init",
@@ -174,16 +247,7 @@ congestion_control_init_params(congestion_control_t *cc,
         CWND_INC_RATE_MIN,
         CWND_INC_RATE_MAX);
 
-#define SENDME_INC_MIN 10
-#define SENDME_INC_MAX (1000)
-  cc->sendme_inc =
-    networkstatus_get_param(NULL, "cc_sendme_inc",
-        sendme_inc,
-        SENDME_INC_MIN,
-        SENDME_INC_MAX);
-
-  // XXX: this min needs to abide by sendme_inc range rules somehow
-#define CWND_MIN_MIN sendme_inc
+#define CWND_MIN_MIN SENDME_INC_DFLT
 #define CWND_MIN_MAX (1000)
   cc->cwnd_min =
     networkstatus_get_param(NULL, "cc_cwnd_min",
@@ -191,29 +255,14 @@ congestion_control_init_params(congestion_control_t *cc,
         CWND_MIN_MIN,
         CWND_MIN_MAX);
 
-#define EWMA_CWND_COUNT_MIN 1
-#define EWMA_CWND_COUNT_MAX (100)
-  cc->ewma_cwnd_cnt =
-    networkstatus_get_param(NULL, "cc_ewma_cwnd_cnt",
-        EWMA_CWND_COUNT_DFLT,
-        EWMA_CWND_COUNT_MIN,
-        EWMA_CWND_COUNT_MAX);
-
-#define BWE_SENDME_MIN_MIN 2
-#define BWE_SENDME_MIN_MAX (20)
-  cc->bwe_sendme_min =
-    networkstatus_get_param(NULL, "cc_bwe_min",
-        BWE_SENDME_MIN_DFLT,
-        BWE_SENDME_MIN_MIN,
-        BWE_SENDME_MIN_MAX);
-
-#define CC_ALG_MIN 0
-#define CC_ALG_MAX (NUM_CC_ALGS-1)
-  cc->cc_alg =
-    networkstatus_get_param(NULL, "cc_alg",
-        cc_alg,
-        CC_ALG_MIN,
-        CC_ALG_MAX);
+  /* If the consensus says to use OG sendme, but torrc has
+   * always-enabled, use the default "always" alg (vegas),
+   * else use cached conensus alg. */
+  if (cc_alg == CC_ALG_SENDME && opts->AlwaysCongestionControl) {
+    cc->cc_alg = CC_ALG_DFLT_ALWAYS;
+  } else {
+    cc->cc_alg = cc_alg;
+  }
 
   bdp_alg_t default_bdp_alg = 0;
 
@@ -243,12 +292,56 @@ congestion_control_init_params(congestion_control_t *cc,
   if (cc->cc_alg == CC_ALG_WESTWOOD) {
     congestion_control_westwood_set_params(cc);
   } else if (cc->cc_alg == CC_ALG_VEGAS) {
-    congestion_control_vegas_set_params(cc);
+    congestion_control_vegas_set_params(cc, path);
   } else if (cc->cc_alg == CC_ALG_NOLA) {
     congestion_control_nola_set_params(cc);
   }
 }
 
+/** Returns true if congestion control is enabled in the most recent
+ * consensus, or if __AlwaysCongestionControl is set to true.
+ *
+ * Note that this function (and many many other functions) should not
+ * be called from the CPU worker threads when handling congestion
+ * control negotiation. Relevant values are marshaled into the
+ * `circuit_params_t` struct, in order to be used in worker threads
+ * without touching global state. Use those values in CPU worker
+ * threads, instead of calling this function.
+ *
+ * The danger is still present, in your time, as it was in ours.
+ */
+bool
+congestion_control_enabled(void)
+{
+  const or_options_t *opts = NULL;
+
+  tor_assert_nonfatal_once(in_main_thread());
+
+  opts = get_options();
+
+  /* If the user has set "__AlwaysCongesttionControl",
+   * then always try to negotiate congestion control, regardless
+   * of consensus param. This is to be used for testing and sbws.
+   *
+   * Note that we do *not* allow disabling congestion control
+   * if the consensus says to use it, as this is bad for queueing
+   * and fairness. */
+  if (opts->AlwaysCongestionControl)
+    return 1;
+
+  return cc_alg != CC_ALG_SENDME;
+}
+
+/**
+ * For unit tests only: set the cached consensus cc alg to
+ * specified value.
+ */
+void
+congestion_control_set_cc_enabled(void)
+{
+  cc_alg = CC_ALG_VEGAS;
+}
+
 /**
  * Allocate and initialize fields in congestion control object.
  *
@@ -258,27 +351,26 @@ congestion_control_init_params(congestion_control_t *cc,
  * acks. This parameter will come from circuit negotiation.
  */
 static void
-congestion_control_init(congestion_control_t *cc, cc_alg_t cc_alg,
-                        int sendme_inc)
+congestion_control_init(congestion_control_t *cc,
+                        const circuit_params_t *params,
+                        cc_path_t path)
 {
   cc->sendme_pending_timestamps = smartlist_new();
   cc->sendme_arrival_timestamps = smartlist_new();
 
   cc->in_slow_start = 1;
-  congestion_control_init_params(cc, cc_alg, sendme_inc);
+  congestion_control_init_params(cc, params, path);
 
   cc->next_cc_event = CWND_UPDATE_RATE(cc);
 }
 
 /** Allocate and initialize a new congestion control object */
 congestion_control_t *
-congestion_control_new(void)
+congestion_control_new(const circuit_params_t *params, cc_path_t path)
 {
   congestion_control_t *cc = tor_malloc_zero(sizeof(congestion_control_t));
 
-  // XXX: the alg and the sendme_inc need to be negotiated during
-  // circuit handshake
-  congestion_control_init(cc, CC_ALG_VEGAS, SENDME_INC_DFLT);
+  congestion_control_init(cc, params, path);
 
   return cc;
 }
@@ -351,14 +443,19 @@ dequeue_timestamp(smartlist_t *timestamps_u64_usecs)
 }
 
 /**
- * Returns the number of sendme acks that will be recieved in the
- * current congestion window size, rounded to nearest int.
+ * Returns the number N of N-count EWMA, for averaging RTT and BDP over
+ * N SENDME acks.
+ *
+ * This N is bracketed between a divisor of the number of acks in a CWND
+ * and a max value. It is always at least 2.
  */
 static inline uint64_t
-sendme_acks_per_cwnd(const congestion_control_t *cc)
+n_ewma_count(const congestion_control_t *cc)
 {
-  /* We add half a sendme_inc to cwnd to round to the nearest int */
-  return ((cc->cwnd + cc->sendme_inc/2)/cc->sendme_inc);
+  uint64_t ewma_cnt = MIN(CWND_UPDATE_RATE(cc)*n_ewma_cwnd_pct/100,
+                          n_ewma_max);
+  ewma_cnt = MAX(ewma_cnt, 2);
+  return ewma_cnt;
 }
 
 /**
@@ -631,7 +728,7 @@ static bool
 time_delta_stalled_or_jumped(const congestion_control_t *cc,
                              uint64_t old_delta, uint64_t new_delta)
 {
-#define DELTA_DISCREPENCY_RATIO_MAX 100
+#define DELTA_DISCREPENCY_RATIO_MAX 5000
   /* If we have a 0 new_delta, that is definitely a monotime stall */
   if (new_delta == 0) {
     static ratelim_t stall_info_limit = RATELIM_INIT(60);
@@ -737,8 +834,7 @@ congestion_control_update_circuit_rtt(congestion_control_t *cc,
     return 0;
   }
 
-  ewma_cnt = cc->ewma_cwnd_cnt*sendme_acks_per_cwnd(cc);
-  ewma_cnt = MAX(ewma_cnt, 2); // Use at least 2
+  ewma_cnt = n_ewma_count(cc);
 
   cc->ewma_rtt_usec = n_count_ewma(rtt, cc->ewma_rtt_usec, ewma_cnt);
 
@@ -854,14 +950,14 @@ congestion_control_update_circuit_bdp(congestion_control_t *cc,
      */
     enqueue_timestamp(cc->sendme_arrival_timestamps, now_usec);
 
-    if (smartlist_len(cc->sendme_arrival_timestamps) >= cc->bwe_sendme_min) {
+    if (smartlist_len(cc->sendme_arrival_timestamps) >= bwe_sendme_min) {
       /* If we have more sendmes than fit in a cwnd, trim the list.
        * Those are not acurrately measuring throughput, if cwnd is
        * currently smaller than BDP */
       while (smartlist_len(cc->sendme_arrival_timestamps) >
-             cc->bwe_sendme_min &&
+             bwe_sendme_min &&
              (uint64_t)smartlist_len(cc->sendme_arrival_timestamps) >
-                       sendme_acks_per_cwnd(cc)) {
+                       n_ewma_count(cc)) {
         (void)dequeue_timestamp(cc->sendme_arrival_timestamps);
       }
       int sendme_cnt = smartlist_len(cc->sendme_arrival_timestamps);
@@ -870,21 +966,26 @@ congestion_control_update_circuit_bdp(congestion_control_t *cc,
       timestamp_usec = peek_timestamp(cc->sendme_arrival_timestamps);
       uint64_t delta = now_usec - timestamp_usec;
 
-      /* The acked data is in sendme_cnt-1 chunks, because we are counting the
-       * data that is processed by the other endpoint *between* all of these
-       * sendmes. There's one less gap between the sendmes than the number
-       * of sendmes. */
-      uint64_t cells = (sendme_cnt-1)*cc->sendme_inc;
+      /* In Shadow, the time delta between acks can be 0 if there is no
+       * network activity between them. Only update BDP if the delta is
+       * non-zero. */
+      if (delta > 0) {
+        /* The acked data is in sendme_cnt-1 chunks, because we are counting
+         * the data that is processed by the other endpoint *between* all of
+         * these sendmes. There's one less gap between the sendmes than the
+         * number of sendmes. */
+        uint64_t cells = (sendme_cnt-1)*cc->sendme_inc;
 
-      /* The bandwidth estimate is cells/delta, which when multiplied
-       * by min RTT obtains the BDP. However, we multiply first to
-       * avoid precision issues with the RTT being close to delta in size. */
-      sendme_rate_bdp = cells*cc->min_rtt_usec/delta;
+        /* The bandwidth estimate is cells/delta, which when multiplied
+         * by min RTT obtains the BDP. However, we multiply first to
+         * avoid precision issues with the RTT being close to delta in size. */
+        sendme_rate_bdp = cells*cc->min_rtt_usec/delta;
 
-      /* Calculate BDP_EWMA_COUNT N-EWMA */
-      cc->bdp[BDP_ALG_SENDME_RATE] =
-                 n_count_ewma(sendme_rate_bdp, cc->bdp[BDP_ALG_SENDME_RATE],
-                              cc->ewma_cwnd_cnt*sendme_acks_per_cwnd(cc));
+        /* Calculate BDP_EWMA_COUNT N-EWMA */
+        cc->bdp[BDP_ALG_SENDME_RATE] =
+                   n_count_ewma(sendme_rate_bdp, cc->bdp[BDP_ALG_SENDME_RATE],
+                                n_ewma_count(cc));
+      }
     }
 
     /* In-flight BDP will cause the cwnd to drift down when underutilized.
@@ -973,9 +1074,8 @@ congestion_control_update_circuit_bdp(congestion_control_t *cc,
                  "%"PRIu64", "
                  "%"PRIu64", "
                  "%"PRIu64". ",
-                 // XXX: actually, is this p_chan here? This is
-                 // an or_circuit (exit or onion)
-                 circ->n_chan->global_identifier, circ->n_circ_id,
+                 CONST_TO_OR_CIRCUIT(circ)->p_chan->global_identifier,
+                 CONST_TO_OR_CIRCUIT(circ)->p_circ_id,
                  cc->min_rtt_usec/1000,
                  curr_rtt_usec/1000,
                  cc->ewma_rtt_usec/1000,
@@ -1036,3 +1136,309 @@ congestion_control_dispatch_cc_alg(congestion_control_t *cc,
 
   return ret;
 }
+
+/**
+ * Build an extension field request to negotiate congestion control.
+ *
+ * If congestion control is enabled, field TRUNNEL_EXT_TYPE_CC_FIELD_REQUEST
+ * is created in msg_out. It is a single 0-length field that signifies that we
+ * want to use congestion control. The length of msg_out is provided via
+ * msg_len_out.
+ *
+ * If congestion control is not enabled, a payload with 0 extensions is created
+ * and returned.
+ *
+ * If there is a failure building the request, -1 is returned, else 0.
+ *
+ * *msg_out must be freed if the return value is 0.
+ */
+int
+congestion_control_build_ext_request(uint8_t **msg_out, size_t *msg_len_out)
+{
+  uint8_t *request = NULL;
+  trn_extension_t *ext = NULL;
+  trn_extension_field_t *field = NULL;
+
+  ext = trn_extension_new();
+
+  /* With congestion control enabled, add the request, else it is an empty
+   * request in the payload. */
+
+  if (congestion_control_enabled()) {
+    /* Build the extension field that will hold the CC field. */
+    field = trn_extension_field_new();
+    trn_extension_field_set_field_type(field,
+                                       TRUNNEL_EXT_TYPE_CC_FIELD_REQUEST);
+
+    /* No payload indicating a request to use congestion control. */
+    trn_extension_field_set_field_len(field, 0);
+
+    /* Build final extension. */
+    trn_extension_add_fields(ext, field);
+    trn_extension_set_num(ext, 1);
+  }
+
+  /* Encode extension. */
+  ssize_t ret = trn_extension_encoded_len(ext);
+  if (BUG(ret < 0)) {
+    goto err;
+  }
+  size_t request_len = ret;
+  request = tor_malloc_zero(request_len);
+  ret = trn_extension_encode(request, request_len, ext);
+  if (BUG(ret < 0)) {
+    tor_free(request);
+    goto err;
+  }
+  *msg_out = request;
+  *msg_len_out = request_len;
+
+  /* Free everything, we've encoded the request now. */
+  ret = 0;
+
+ err:
+  trn_extension_free(ext);
+  return (int)ret;
+}
+
+/**
+ * Parse a congestion control ntorv3 request payload for extensions.
+ *
+ * On parsing failure, -1 is returned.
+ *
+ * If congestion control request is present, return 1. If it is not present,
+ * return 0.
+ *
+ * WARNING: Called from CPU worker! Must not access any global state.
+ */
+int
+congestion_control_parse_ext_request(const uint8_t *msg, const size_t msg_len)
+{
+  ssize_t ret = 0;
+  trn_extension_t *ext = NULL;
+  size_t num_fields = 0;
+
+  /* Parse extension from payload. */
+  ret = trn_extension_parse(&ext, msg, msg_len);
+  if (ret < 0) {
+    goto end;
+  }
+
+  /* No extension implies no support for congestion control. In this case, we
+   * simply return 0 to indicate CC is disabled. */
+  if ((num_fields = trn_extension_get_num(ext)) == 0) {
+    ret = 0;
+    goto end;
+  }
+
+  /* Go over all fields. If any field is TRUNNEL_EXT_TYPE_CC_FIELD_REQUEST,
+   * then congestion control is enabled. Ignore unknown fields. */
+  for (size_t f = 0; f < num_fields; f++) {
+    const trn_extension_field_t *field = trn_extension_get_fields(ext, f);
+    if (field == NULL) {
+      ret = -1;
+      goto end;
+    }
+
+    /* For congestion control to be enabled, we only need the field type. */
+    if (trn_extension_field_get_field_type(field) ==
+        TRUNNEL_EXT_TYPE_CC_FIELD_REQUEST) {
+      ret = 1;
+      break;
+    }
+  }
+
+ end:
+  trn_extension_free(ext);
+  return (int)ret;
+}
+
+/**
+ * Given our observed parameters for circuits and congestion control,
+ * as well as the parameters for the resulting circuit, build a response
+ * payload using extension fields into *msg_out, with length specified in
+ * *msg_out_len.
+ *
+ * If congestion control will be enabled, the extension field for
+ * TRUNNEL_EXT_TYPE_CC_FIELD_RESPONSE will contain the sendme_inc value.
+ *
+ * If congestion control won't be enabled, an extension payload with 0
+ * fields will be created.
+ *
+ * Return 0 if an extension payload was created in *msg_out, and -1 on
+ * error.
+ *
+ * *msg_out must be freed if the return value is 0.
+ *
+ * WARNING: Called from CPU worker! Must not access any global state.
+ */
+int
+congestion_control_build_ext_response(const circuit_params_t *our_params,
+                                      const circuit_params_t *circ_params,
+                                      uint8_t **msg_out, size_t *msg_len_out)
+{
+  ssize_t ret;
+  uint8_t *request = NULL;
+  trn_extension_t *ext = NULL;
+  trn_extension_field_t *field = NULL;
+  trn_extension_field_cc_t *cc_field = NULL;
+
+  tor_assert(our_params);
+  tor_assert(circ_params);
+  tor_assert(msg_out);
+  tor_assert(msg_len_out);
+
+  ext = trn_extension_new();
+
+  if (circ_params->cc_enabled) {
+    /* Build the extension field that will hold the CC field. */
+    field = trn_extension_field_new();
+    trn_extension_field_set_field_type(field,
+                                       TRUNNEL_EXT_TYPE_CC_FIELD_RESPONSE);
+
+    /* Build the congestion control field response. */
+    cc_field = trn_extension_field_cc_new();
+    trn_extension_field_cc_set_sendme_inc(cc_field,
+                                          our_params->sendme_inc_cells);
+
+    ret = trn_extension_field_cc_encoded_len(cc_field);
+    if (BUG(ret <= 0)) {
+      trn_extension_field_free(field);
+      goto err;
+    }
+    size_t field_len = ret;
+    trn_extension_field_set_field_len(field, field_len);
+    trn_extension_field_setlen_field(field, field_len);
+
+    uint8_t *field_array = trn_extension_field_getarray_field(field);
+    ret = trn_extension_field_cc_encode(field_array,
+              trn_extension_field_getlen_field(field), cc_field);
+    if (BUG(ret <= 0)) {
+      trn_extension_field_free(field);
+      goto err;
+    }
+
+    /* Build final extension. */
+    trn_extension_add_fields(ext, field);
+    trn_extension_set_num(ext, 1);
+  }
+
+  /* Encode extension. */
+  ret = trn_extension_encoded_len(ext);
+  if (BUG(ret < 0)) {
+    goto err;
+  }
+  size_t request_len = ret;
+  request = tor_malloc_zero(request_len);
+  ret = trn_extension_encode(request, request_len, ext);
+  if (BUG(ret < 0)) {
+    tor_free(request);
+    goto err;
+  }
+  *msg_out = request;
+  *msg_len_out = request_len;
+
+  /* We've just encoded the extension, clean everything. */
+  ret = 0;
+
+ err:
+  trn_extension_free(ext);
+  trn_extension_field_cc_free(cc_field);
+  return (int)ret;
+}
+
+/** Return true iff the given sendme increment is within the acceptable
+ * margins. */
+bool
+congestion_control_validate_sendme_increment(uint8_t sendme_inc)
+{
+  /* We will only accept this response (and this circuit) if sendme_inc
+   * is within a factor of 2 of our consensus value. We should not need
+   * to change cc_sendme_inc much, and if we do, we can spread out those
+   * changes over smaller increments once every 4 hours. Exits that
+   * violate this range should just not be used. */
+#define MAX_SENDME_INC_NEGOTIATE_FACTOR 2
+
+  if (sendme_inc == 0)
+    return false;
+
+  if (sendme_inc >
+      MAX_SENDME_INC_NEGOTIATE_FACTOR * congestion_control_sendme_inc() ||
+      sendme_inc <
+      congestion_control_sendme_inc() / MAX_SENDME_INC_NEGOTIATE_FACTOR) {
+    return false;
+  }
+  return true;
+}
+
+/** Return 1 if CC is enabled which also will set the SENDME increment into our
+ * params_out. Return 0 if CC is disabled. Else, return -1 on error. */
+int
+congestion_control_parse_ext_response(const uint8_t *msg,
+                                      const size_t msg_len,
+                                      circuit_params_t *params_out)
+{
+  ssize_t ret = 0;
+  size_t num_fields = 0;
+  trn_extension_t *ext = NULL;
+  trn_extension_field_cc_t *cc_field = NULL;
+
+  /* We will only accept this response (and this circuit) if sendme_inc
+   * is within a factor of 2 of our consensus value. We should not need
+   * to change cc_sendme_inc much, and if we do, we can spread out those
+   * changes over smaller increments once every 4 hours. Exits that
+   * violate this range should just not be used. */
+#define MAX_SENDME_INC_NEGOTIATE_FACTOR 2
+
+  /* Parse extension from payload. */
+  ret = trn_extension_parse(&ext, msg, msg_len);
+  if (ret < 0) {
+    goto end;
+  }
+
+  if ((num_fields = trn_extension_get_num(ext)) == 0) {
+    ret = 0;
+    goto end;
+  }
+
+  /* Go over all fields. If any field is TRUNNEL_EXT_TYPE_CC_FIELD_RESPONSE,
+   * then congestion control is enabled. Ignore unknown fields. */
+  for (size_t f = 0; f < num_fields; f++) {
+    const trn_extension_field_t *field = trn_extension_get_fields(ext, f);
+    if (field == NULL) {
+      ret = -1;
+      goto end;
+    }
+
+    /* Only examine TRUNNEL_EXT_TYPE_CC_FIELD_RESPONSE; ignore other fields */
+    if (trn_extension_field_get_field_type(field) ==
+        TRUNNEL_EXT_TYPE_CC_FIELD_RESPONSE) {
+
+      /* Parse the field into the congestion control field. */
+      ret = trn_extension_field_cc_parse(&cc_field,
+                trn_extension_field_getconstarray_field(field),
+                trn_extension_field_getlen_field(field));
+      if (ret < 0) {
+        goto end;
+      }
+
+      uint8_t sendme_inc_cells =
+              trn_extension_field_cc_get_sendme_inc(cc_field);
+      if (!congestion_control_validate_sendme_increment(sendme_inc_cells)) {
+        ret = -1;
+        goto end;
+      }
+
+      /* All good. Get value and break */
+      params_out->sendme_inc_cells = sendme_inc_cells;
+      ret = 1;
+      break;
+    }
+  }
+
+ end:
+  trn_extension_free(ext);
+  trn_extension_field_cc_free(cc_field);
+
+  return (int)ret;
+}
