@@ -9,18 +9,41 @@
 #ifndef TOR_CONGESTION_CONTROL_COMMON_H
 #define TOR_CONGESTION_CONTROL_COMMON_H
 
+#include "core/crypto/onion_crypto.h"
 #include "core/or/crypt_path_st.h"
 #include "core/or/circuit_st.h"
 
+/* The maximum whole number of cells that can fit in a
+ * full TLS record. This is 31. */
+#define TLS_RECORD_MAX_CELLS ((16 * 1024) / CELL_MAX_NETWORK_SIZE)
+
 typedef struct congestion_control_t congestion_control_t;
 
+/**
+ * Specifies the path type to help choose congestion control
+ * parameters. Since these paths are different lengths, they
+ * will need different queue parameters. */
+typedef enum {
+  CC_PATH_EXIT = 0,
+  CC_PATH_ONION = 1,
+  CC_PATH_ONION_SOS = 2,
+  CC_PATH_ONION_VG = 3,
+  CC_PATH_SBWS = 4,
+} cc_path_t;
+
+/** The length of a path for sbws measurement */
+#define SBWS_ROUTE_LEN 2
+
 /** Wrapper for the free function, set the CC pointer to NULL after free */
 #define congestion_control_free(cc) \
     FREE_AND_NULL(congestion_control_t, congestion_control_free_, cc)
 
 void congestion_control_free_(congestion_control_t *cc);
 
-congestion_control_t *congestion_control_new(void);
+struct circuit_params_t;
+congestion_control_t *congestion_control_new(
+                                    const struct circuit_params_t *params,
+                                    cc_path_t path);
 
 int congestion_control_dispatch_cc_alg(congestion_control_t *cc,
                                        const circuit_t *circ,
@@ -43,12 +66,28 @@ bool is_monotime_clock_reliable(void);
 
 void congestion_control_new_consensus_params(const networkstatus_t *ns);
 
-/* Ugh, C.. these four are private. Use the getter instead, when
+bool congestion_control_enabled(void);
+
+int congestion_control_build_ext_request(uint8_t **msg_out,
+                                         size_t *msg_len_out);
+int congestion_control_parse_ext_request(const uint8_t *msg,
+                                         const size_t msg_len);
+int congestion_control_build_ext_response(const circuit_params_t *our_params,
+                                          const circuit_params_t *circ_params,
+                                          uint8_t **msg_out,
+                                          size_t *msg_len_out);
+int congestion_control_parse_ext_response(const uint8_t *msg,
+                                          const size_t msg_len,
+                                          circuit_params_t *params_out);
+bool congestion_control_validate_sendme_increment(uint8_t sendme_inc);
+
+/* Ugh, C.. these are private. Use the getter instead, when
  * external to the congestion control code. */
 extern uint32_t or_conn_highwater;
 extern uint32_t or_conn_lowwater;
 extern int32_t cell_queue_high;
 extern int32_t cell_queue_low;
+extern uint8_t cc_sendme_inc;
 
 /** Stop writing on an orconn when its outbuf is this large */
 static inline uint32_t
@@ -80,6 +119,13 @@ cell_queue_lowwatermark(void)
   return cell_queue_low;
 }
 
+/** Returns the sendme inc rate cached from the most recent consensus */
+static inline uint8_t
+congestion_control_sendme_inc(void)
+{
+  return cc_sendme_inc;
+}
+
 /**
  * Compute an N-count EWMA, aka N-EWMA. N-EWMA is defined as:
  *  EWMA = alpha*value + (1-alpha)*EWMA_prev
@@ -106,6 +152,8 @@ n_count_ewma(uint64_t curr, uint64_t prev, uint64_t N)
  */
 #ifdef TOR_UNIT_TESTS
 
+void congestion_control_set_cc_enabled(void);
+
 #endif /* defined(TOR_UNIT_TESTS) */
 
 #endif /* defined(TOR_CONGESTION_CONTROL_PRIVATE) */
