@@ -62,12 +62,15 @@ static uint32_t xon_rate_bytes;
 static inline const congestion_control_t *
 edge_get_ccontrol(const edge_connection_t *edge)
 {
-  if (edge->cpath_layer)
-    return edge->cpath_layer->ccontrol;
-  else if (edge->on_circuit)
-    return edge->on_circuit->ccontrol;
-  else
-    return NULL;
+  congestion_control_t *ccontrol = NULL;
+
+  if (edge->on_circuit && edge->on_circuit->ccontrol) {
+    ccontrol = edge->on_circuit->ccontrol;
+  } else if (edge->cpath_layer && edge->cpath_layer->ccontrol) {
+    ccontrol = edge->cpath_layer->ccontrol;
+  }
+
+  return ccontrol;
 }
 
 /**
@@ -113,7 +116,7 @@ flow_control_new_consensus_params(const networkstatus_t *ns)
       CC_XON_RATE_BYTES_MAX)*RELAY_PAYLOAD_SIZE;
 
 #define CC_XON_EWMA_CNT_DFLT (2)
-#define CC_XON_EWMA_CNT_MIN (1)
+#define CC_XON_EWMA_CNT_MIN (2)
 #define CC_XON_EWMA_CNT_MAX (100)
   xon_ewma_cnt = networkstatus_get_param(ns, "cc_xon_ewma_cnt",
       CC_XON_EWMA_CNT_DFLT,
@@ -275,10 +278,6 @@ circuit_process_stream_xoff(edge_connection_t *conn,
    */
   if (TO_CONN(conn)->type == CONN_TYPE_AP || conn->hs_ident != NULL) {
     uint32_t limit = 0;
-
-    /* TODO: This limit technically needs to come from negotiation,
-     * and be bounds checked for sanity, because the other endpoint
-     * may have a different consensus */
     if (conn->hs_ident)
       limit = xoff_client;
     else
@@ -296,9 +295,6 @@ circuit_process_stream_xoff(edge_connection_t *conn,
     }
   }
 
-  // TODO: Count how many xoffs we have; log if "too many", for shadow
-  // analysis of chatter. Possibly add to extra-info?
-
   log_info(LD_EDGE, "Got XOFF!");
   connection_stop_reading(TO_CONN(conn));
   conn->xoff_received = true;
@@ -371,9 +367,6 @@ circuit_process_stream_xon(edge_connection_t *conn,
   if (TO_CONN(conn)->type == CONN_TYPE_AP || conn->hs_ident != NULL) {
     uint32_t limit = 0;
 
-    /* TODO: This limit technically needs to come from negotiation,
-     * and be bounds checked for sanity, because the other endpoint
-     * may have a different consensus */
     if (conn->hs_ident)
       limit = MIN(xoff_client, xon_rate_bytes);
     else
