@@ -1,4 +1,4 @@
-/* Copyright (c) 2016-2020, The Tor Project, Inc. */
+/* Copyright (c) 2016-2021, The Tor Project, Inc. */
 /* See LICENSE for licensing information */
 
 /**
@@ -11,12 +11,15 @@
 #include "core/or/or.h"
 #include "app/config/config.h"
 #include "core/crypto/hs_ntor.h"
+#include "core/crypto/onion_crypto.h"
 #include "core/mainloop/connection.h"
 #include "core/or/circuitbuild.h"
 #include "core/or/circuitlist.h"
 #include "core/or/circuituse.h"
 #include "core/or/connection_edge.h"
+#include "core/or/congestion_control_common.h"
 #include "core/or/extendinfo.h"
+#include "core/or/protover.h"
 #include "core/or/reasons.h"
 #include "feature/client/circpathbias.h"
 #include "feature/dirclient/dirclient.h"
@@ -30,10 +33,10 @@
 #include "feature/hs/hs_descriptor.h"
 #include "feature/hs/hs_ident.h"
 #include "feature/nodelist/describe.h"
+#include "feature/nodelist/microdesc.h"
 #include "feature/nodelist/networkstatus.h"
 #include "feature/nodelist/nodelist.h"
 #include "feature/nodelist/routerset.h"
-#include "feature/rend/rendclient.h"
 #include "lib/crypt_ops/crypto_format.h"
 #include "lib/crypt_ops/crypto_rand.h"
 #include "lib/crypt_ops/crypto_util.h"
@@ -359,16 +362,6 @@ note_connection_attempt_succeeded(const hs_ident_edge_conn_t *hs_conn_ident)
   /* Remove from the hid serv cache all requests for that service so we can
    * query the HSDir again later on for various reasons. */
   purge_hid_serv_request(&hs_conn_ident->identity_pk);
-
-  /* The v2 subsystem cleans up the intro point time out flag at this stage.
-   * We don't try to do it here because we still need to keep intact the intro
-   * point state for future connections. Even though we are able to connect to
-   * the service, doesn't mean we should reset the timed out intro points.
-   *
-   * It is not possible to have successfully connected to an intro point
-   * present in our cache that was on error or timed out. Every entry in that
-   * cache have a 2 minutes lifetime so ultimately the intro point(s) state
-   * will be reset and thus possible to be retried. */
 }
 
 /** Given the pubkey of a hidden service in <b>onion_identity_pk</b>, fetch its
@@ -766,6 +759,51 @@ client_intro_circ_has_opened(origin_circuit_t *circ)
   connection_ap_attach_pending(1);
 }
 
+/** Setup the congestion control parameters on the given rendezvous circuit.
+ * This looks at the service descriptor flow control line (if any). */
+static void
+setup_rendezvous_circ_congestion_control(origin_circuit_t *circ)
+{
+  circuit_params_t circ_params = {0};
+
+  tor_assert(circ);
+
+  /* Setup congestion control parameters on the circuit. */
+  const hs_descriptor_t *desc =
+    hs_cache_lookup_as_client(&circ->hs_ident->identity_pk);
+  if (BUG(desc == NULL)) {
+    /* This should really never happened but in case, scream and stop. */
+    return;
+  }
+
+  /* Check if the service lists support for congestion control in its
+   * descriptor. If not, we don't setup congestion control. */
+  if (!desc->encrypted_data.flow_control_pv ||
+      !protocol_list_supports_protocol(desc->encrypted_data.flow_control_pv,
+                                       PRT_FLOWCTRL, PROTOVER_FLOWCTRL_CC)) {
+    return;
+  }
+
+  /* Take values from the consensus. */
+  circ_params.cc_enabled = congestion_control_enabled();
+  if (circ_params.cc_enabled) {
+    circ_params.sendme_inc_cells = desc->encrypted_data.sendme_inc;
+
+    if (desc->encrypted_data.single_onion_service) {
+      TO_CIRCUIT(circ)->ccontrol = congestion_control_new(&circ_params,
+                                                          CC_PATH_ONION_SOS);
+    } else {
+      if (get_options()->HSLayer3Nodes) {
+        TO_CIRCUIT(circ)->ccontrol = congestion_control_new(&circ_params,
+                                                            CC_PATH_ONION_VG);
+      } else {
+        TO_CIRCUIT(circ)->ccontrol = congestion_control_new(&circ_params,
+                                                            CC_PATH_ONION);
+      }
+    }
+  }
+}
+
 /** Called when a rendezvous circuit has opened. */
 static void
 client_rendezvous_circ_has_opened(origin_circuit_t *circ)
@@ -795,6 +833,9 @@ client_rendezvous_circ_has_opened(origin_circuit_t *circ)
   log_info(LD_REND, "Rendezvous circuit has opened to %s.",
            safe_str_client(extend_info_describe(rp_ei)));
 
+  /* Setup congestion control parameters on the circuit. */
+  setup_rendezvous_circ_congestion_control(circ);
+
   /* Ignore returned value, nothing we can really do. On failure, the circuit
    * will be marked for close. */
   hs_circ_send_establish_rendezvous(circ);
@@ -1130,7 +1171,7 @@ handle_introduce_ack_success(origin_circuit_t *intro_circ)
   rend_circ =
   hs_circuitmap_get_established_rend_circ_client_side(rendezvous_cookie);
   if (rend_circ == NULL) {
-    log_warn(LD_REND, "Can't find any rendezvous circuit. Stopping");
+    log_info(LD_REND, "Can't find any rendezvous circuit. Stopping");
     goto end;
   }
 
@@ -1303,9 +1344,10 @@ can_client_refetch_desc(const ed25519_public_key_t *identity_pk,
     goto cannot;
   }
 
-  /* Without a live consensus we can't do any client actions. It is needed to
-   * compute the hashring for a service. */
-  if (!networkstatus_get_live_consensus(approx_time())) {
+  /* Without a usable consensus we can't do any client actions. It is needed
+   * to compute the hashring for a service. */
+  if (!networkstatus_get_reasonably_live_consensus(approx_time(),
+                                         usable_consensus_flavor())) {
     log_info(LD_REND, "Can't fetch descriptor for service %s because we "
                       "are missing a live consensus. Stalling connection.",
              safe_str_client(ed25519_fmt(identity_pk)));
@@ -1949,17 +1991,9 @@ hs_client_note_connection_attempt_succeeded(const edge_connection_t *conn)
 {
   tor_assert(connection_edge_is_rendezvous_stream(conn));
 
-  if (BUG(conn->rend_data && conn->hs_ident)) {
-    log_warn(LD_BUG, "Stream had both rend_data and hs_ident..."
-             "Prioritizing hs_ident");
-  }
-
   if (conn->hs_ident) { /* It's v3: pass it to the prop224 handler */
     note_connection_attempt_succeeded(conn->hs_ident);
     return;
-  } else if (conn->rend_data) { /* It's v2: pass it to the legacy handler */
-    rend_client_note_connection_attempt_ended(conn->rend_data);
-    return;
   }
 }
 
@@ -2085,9 +2119,7 @@ int
 hs_client_send_introduce1(origin_circuit_t *intro_circ,
                           origin_circuit_t *rend_circ)
 {
-  return (intro_circ->hs_ident) ? send_introduce1(intro_circ, rend_circ) :
-                                  rend_client_send_introduction(intro_circ,
-                                                                rend_circ);
+  return send_introduce1(intro_circ, rend_circ);
 }
 
 /** Called when the client circuit circ has been established. It can be either
@@ -2098,21 +2130,15 @@ hs_client_circuit_has_opened(origin_circuit_t *circ)
 {
   tor_assert(circ);
 
-  /* Handle both version. v2 uses rend_data and v3 uses the hs circuit
-   * identifier hs_ident. Can't be both. */
   switch (TO_CIRCUIT(circ)->purpose) {
   case CIRCUIT_PURPOSE_C_INTRODUCING:
     if (circ->hs_ident) {
       client_intro_circ_has_opened(circ);
-    } else {
-      rend_client_introcirc_has_opened(circ);
     }
     break;
   case CIRCUIT_PURPOSE_C_ESTABLISH_REND:
     if (circ->hs_ident) {
       client_rendezvous_circ_has_opened(circ);
-    } else {
-      rend_client_rendcirc_has_opened(circ);
     }
     break;
   default:
@@ -2426,9 +2452,7 @@ hs_client_get_random_intro_from_edge(const edge_connection_t *edge_conn)
 {
   tor_assert(edge_conn);
 
-  return (edge_conn->hs_ident) ?
-    client_get_random_intro(&edge_conn->hs_ident->identity_pk) :
-    rend_client_get_random_intro(edge_conn->rend_data);
+  return client_get_random_intro(&edge_conn->hs_ident->identity_pk);
 }
 
 /** Called when get an INTRODUCE_ACK cell on the introduction circuit circ.
@@ -2450,9 +2474,7 @@ hs_client_receive_introduce_ack(origin_circuit_t *circ,
     goto end;
   }
 
-  ret = (circ->hs_ident) ? handle_introduce_ack(circ, payload, payload_len) :
-                           rend_client_introduction_acked(circ, payload,
-                                                          payload_len);
+  ret = handle_introduce_ack(circ, payload, payload_len);
   /* For path bias: This circuit was used successfully. NACK or ACK counts. */
   pathbias_mark_use_success(circ);
 
@@ -2486,9 +2508,8 @@ hs_client_receive_rendezvous2(origin_circuit_t *circ,
   log_info(LD_REND, "Got RENDEZVOUS2 cell from hidden service on circuit %u.",
            TO_CIRCUIT(circ)->n_circ_id);
 
-  ret = (circ->hs_ident) ? handle_rendezvous2(circ, payload, payload_len) :
-                           rend_client_receive_rendezvous(circ, payload,
-                                                          payload_len);
+  ret = handle_rendezvous2(circ, payload, payload_len);
+
  end:
   return ret;
 }
@@ -2509,9 +2530,7 @@ hs_client_reextend_intro_circuit(origin_circuit_t *circ)
 
   tor_assert(circ);
 
-  ei = (circ->hs_ident) ?
-    client_get_random_intro(&circ->hs_ident->identity_pk) :
-    rend_client_get_random_intro(circ->rend_data);
+  ei = client_get_random_intro(&circ->hs_ident->identity_pk);
   if (ei == NULL) {
     log_warn(LD_REND, "No usable introduction points left. Closing.");
     circuit_mark_for_close(TO_CIRCUIT(circ), END_CIRC_REASON_INTERNAL);
@@ -2589,9 +2608,6 @@ hs_client_free_all(void)
 void
 hs_client_purge_state(void)
 {
-  /* v2 subsystem. */
-  rend_client_purge_state();
-
   /* Cancel all descriptor fetches. Do this first so once done we are sure
    * that our descriptor cache won't modified. */
   cancel_descriptor_fetches();
